#!/opt/pmx5/bin/perl

use strict;
use warnings;

use HTML::Template;
use PureMessage::Service;
use PureMessage::Scheduler::Config;
use PureMessage::Manager::Request qw($REQUEST %in %text);
use PureMessage::Manager::Response::Services;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::InfoBox;
use PureMessage::Manager::Lang;
use PureMessage::Service::Config qw(stale);
use PureMessage::Version qw($PREFIX);

my $FILE_TMPL = "tmpl/ip_blocking.html";
my $DATA_UPDATE_EVENT = 'pmx-blocklist-data-update';
my $CONSUME_BLOCKLIST_LOG_EVENT = 'consume-blocklist-log';

my $WIDGET = new PureMessage::Manager::Widget;

my $resp = $REQUEST->get_response( 'PureMessage::Manager::Response::Services',
                                    'stderr_to_stdout' => 1,
                                  );

$resp->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_ip_blocking" );

# get available services and check if the blocker is registered
my @services = PureMessage::Service->Available;

my ($blocker, $scheduler, $smtp) = ();

foreach my $s (@services) {
    if ($s->type eq 'blockerd') {
        $blocker = $s;
    }
    elsif ($s->type eq 'scheduler') {
        $scheduler = $s;
    }
    elsif ($s->type eq 'smtp') {
        $smtp = $s;
    }
}

my $msg;

# print warning popup if the blocker service is not available
if (not $blocker) {
    $msg = "IP Blocker Service is not installed! Please make sure that the service
            is available prior to accessing functionality related to it.";
}
elsif (not $smtp) {
    $msg = "MTA IP Blocking service can not be configured because no local MTA has
            been detected.";
}
elsif (not $scheduler) {
    $msg = "The Scheduler service is not installed! Please check your
	    PureMessage Installation";
}

if ($msg) {
    $resp->addInfoBox(msg => $msg);
    $resp->send();
    exit;
}

# the service is available, send the page to the client
$resp->title(text('ip_blocking_title'));

my $blocklist_is_enabled = get_blocklist_status();

# enable blocking service only if it hasn't previously been enabled
# (this should prevent unnecessary calls to the enable_ip_blocking sub
if (exists $in{'ip_block'}) {
    if (($in{'ip_block'} eq 'enable') and not $blocklist_is_enabled) {
        
        my $rc = enable_ip_blocking();
        $blocklist_is_enabled = get_blocklist_status();

        # print warning msg and enable the scheduler job if it isn't done already
        if ($blocklist_is_enabled) {
            my $sc = PureMessage::Scheduler::Config->new(writable => 1);
            my $config_changed = 0;
            
            if (not $sc->is_enabled_event($DATA_UPDATE_EVENT)) {
                $msg = "When enabling or re-enabling MTA level IP blocking, PureMessage
                        fetches a copy of the current blocklist data package
                        (PureMessage-Blocklist-Data) which can be several MB in size.
                        This data is updated periodically by a Scheduler Service job.";
                $resp->addInfoBox(msg => $msg);
                $sc->enable_event($DATA_UPDATE_EVENT);
                $config_changed = 1;
            
		# run the data uppdate job right away, rather than waiting
		eval {
		    $scheduler->trigger(event => $DATA_UPDATE_EVENT);
		    $msg = "The initial blocklist data package is being downloaded now.";
		    $resp->addInfoBox(msg => $msg);
		};
            }
            
            if (not $sc->is_enabled_event($CONSUME_BLOCKLIST_LOG_EVENT)) {
                $sc->enable_event($CONSUME_BLOCKLIST_LOG_EVENT);
                $config_changed =1;
            }
      
            $sc->save() if ($config_changed);
            
            # mark the smtp service as stale
            stale($smtp->type(), $smtp->name()) if $smtp;
            
        }
        else {
            $msg = "Failed to enable the blocklist service! Please check your system";
            $resp->addInfoBox(msg => $msg);
        }
    }
}
# disable the service only if enabled
elsif ($blocklist_is_enabled and not defined $in{'ip_block'} and defined $in{'sm_save'}) {
    
    my $rc = disable_ip_blocking();
    $blocklist_is_enabled = get_blocklist_status();
        
    if ($blocklist_is_enabled) {
        $msg = "Failed to disable the blocklist service! Please check your system";
        $resp->addInfoBox(msg => $msg);
    }
    else {
        # mark the smtp service as stale
        stale($smtp->type(), $smtp->name()) if $smtp;
    }
}

$resp->addRestartDlg(query_ok => 1);

# display the page with the current status of the service
my $template = HTML::Template->new( 'filename' => $FILE_TMPL );

if ($blocklist_is_enabled) {
    $template->param(check => 'CHECKED');
}
else {
    $template->param(check => '');
}

my $table_format = { header => $WIDGET->section_header('IP Blocking Service options'),
                     footer => $resp->ThemeSectionFooter(
                                toolbar => $resp->ToolBar (
                                    buttons => [[ { type=>'image',
                                                    icon=>'save',
                                                    alt=>'Save',
                                                    text=>'Save',
                                                    name=>'sm_save',
                                                  },
                                               ]],
                                               )
                                                       )->output(),
                  };

$template->param($table_format);
$resp->add_part($template);

$resp->send();


sub get_blocklist_status {
    return $blocker->service_status();
}

sub enable_ip_blocking {
    return $blocker->enable();
}

sub disable_ip_blocking {
    return $blocker->disable();
}

